@import url(http://fonts.googleapis.com/css?family=Merriweather:300italic,300);

html {
    font-size: 100%;
    overflow-y: scroll;
    -webkit-text-size-adjust: 100%;
    -ms-text-size-adjust: 100%;
    zoom: 67%;
}

body {
  background: #aaa;
  font-family: 'Merriweather', Georgia, serif;
  font-size: 18px;
}

.container {
  margin: auto;
  width: 80%;
  background: #d0d0d0;
  padding: 10px;
  border-radius: 25px;
  border-right:  1px solid #aaa;
  border-left:   1px solid #aaa;
  box-shadow: 0 30px 40px rgba(0,0,0,.1);
}

.date {
  color: #aaa;
}


.entry {
  margin: auto;
  width: 80%;
  background: #ddd;
  padding: 20px;
  border-radius: 5px;
  border-bottom: 1px solid #aaa;
  overflow-x: auto;
}
.entry_spacer {
  margin: auto;
  width: 80%;
  padding: 10px;
}

.entry pre,
.entry code {
  font-family: Menlo, Monaco, "Courier New", monospace;
  border-radius: 5px;
}
.entry .code {
  width: 100%;
}
.entry pre {
  padding: .5rem;
  line-height: 1.25;
  overflow-x: scroll;
}
.entry pre,
.entry blockquote {
  page-break-inside: avoid;
}
hr {
    border: 1px solid #bbb;
}
.entry blockquote {
    border-left: 3px solid #bbb;
    white-space: pre-line;
    padding-left: 1rem;
}

.entry a {
  font-family: Menlo, Monaco, "Courier New", monospace;
  font-weight: bold;
}
.entry a:link { color: #555; }
.entry a:visited { color: #888; }
.entry a:hover, .entry a:focus, .entry a:active { color:  #111; }

.entry table,
.entry th,
.entry td {

font-family: Menlo, Monaco, "Courier New", monospace;
  border: 1px solid #bbb;
  border-collapse: collapse;
}
.entry thead tr {
  background-color: #e6e6e6;
}
.entry table :is(td, th) {
  padding: 0.3em;
}
.entry tbody tr:nth-child(even) {
    background-color: #e6e6e6;
}
.entry tbody tr:nth-child(odd) {
    background-color: #ccc;
}


.codehilite .hll { background-color: #ffffcc }
.codehilite  { background: #e6e6e6; }
.codehilite .c { color: #008000 } /* Comment */
.codehilite .err { border: 1px solid #FF0000 } /* Error */
.codehilite .k { color: #0000ff } /* Keyword */
.codehilite .ch { color: #008000 } /* Comment.Hashbang */
.codehilite .cm { color: #008000 } /* Comment.Multiline */
.codehilite .cp { color: #0000ff } /* Comment.Preproc */
.codehilite .cpf { color: #008000 } /* Comment.PreprocFile */
.codehilite .c1 { color: #008000 } /* Comment.Single */
.codehilite .cs { color: #008000 } /* Comment.Special */
.codehilite .ge { font-style: italic } /* Generic.Emph */
.codehilite .gh { font-weight: bold } /* Generic.Heading */
.codehilite .gp { font-weight: bold } /* Generic.Prompt */
.codehilite .gs { font-weight: bold } /* Generic.Strong */
.codehilite .gu { font-weight: bold } /* Generic.Subheading */
.codehilite .kc { color: #0000ff } /* Keyword.Constant */
.codehilite .kd { color: #0000ff } /* Keyword.Declaration */
.codehilite .kn { color: #0000ff } /* Keyword.Namespace */
.codehilite .kp { color: #0000ff } /* Keyword.Pseudo */
.codehilite .kr { color: #0000ff } /* Keyword.Reserved */
.codehilite .kt { color: #2b91af } /* Keyword.Type */
.codehilite .s { color: #a31515 } /* Literal.String */
.codehilite .nc { color: #2b91af } /* Name.Class */
.codehilite .ow { color: #0000ff } /* Operator.Word */
.codehilite .sa { color: #a31515 } /* Literal.String.Affix */
.codehilite .sb { color: #a31515 } /* Literal.String.Backtick */
.codehilite .sc { color: #a31515 } /* Literal.String.Char */
.codehilite .dl { color: #a31515 } /* Literal.String.Delimiter */
.codehilite .sd { color: #a31515 } /* Literal.String.Doc */
.codehilite .s2 { color: #a31515 } /* Literal.String.Double */
.codehilite .se { color: #a31515 } /* Literal.String.Escape */
.codehilite .sh { color: #a31515 } /* Literal.String.Heredoc */
.codehilite .si { color: #a31515 } /* Literal.String.Interpol */
.codehilite .sx { color: #a31515 } /* Literal.String.Other */
.codehilite .sr { color: #a31515 } /* Literal.String.Regex */
.codehilite .s1 { color: #a31515 } /* Literal.String.Single */
.codehilite .ss { color: #a31515 } /* Literal.String.Symbol */